/// <reference types="jest" />
/**
 * This is the core of settings and so ts-jest.
 * Since configuration are used to create a good cache key, everything
 * depending on it is here. Fast jest relies on correct cache keys
 * depending on all settings that could affect the generated output.
 *
 * The big issue is that jest calls first `getCacheKey()` with stringified
 * version of the `jest.ProjectConfig`, and then later it calls `process()`
 * with the complete, object version of it.
 */
import { JsonableValue } from '../util/jsonable-value';
import { BabelConfig, TsJestConfig, TsJestGlobalOptions, TTypeScript, TsJestHooksMap, BabelJestTransformer, TsCompiler } from '../types';
import { Diagnostic, ParsedCommandLine } from 'typescript';
import { TSError } from '../util/ts-error';
import { Logger } from 'bs-logger';
interface ReadTsConfigResult {
    input?: any;
    resolved: ParsedCommandLine;
}
export declare const MATCH_NOTHING: RegExp;
export declare const IGNORE_DIAGNOSTIC_CODES: number[];
export declare class ConfigSet {
    private _jestConfig;
    readonly parentOptions?: TsJestGlobalOptions | undefined;
    readonly logger: Logger;
    constructor(_jestConfig: jest.ProjectConfig, parentOptions?: TsJestGlobalOptions | undefined, parentLogger?: Logger);
    readonly jest: jest.ProjectConfig;
    readonly tsJest: TsJestConfig;
    readonly typescript: ParsedCommandLine;
    readonly tsconfig: any;
    private readonly _typescript;
    readonly babel: BabelConfig | undefined;
    readonly compilerModule: TTypeScript;
    readonly babelJestTransformer: BabelJestTransformer | undefined;
    readonly tsCompiler: TsCompiler;
    readonly hooks: TsJestHooksMap;
    readonly filterDiagnostics: (diagnostics: Diagnostic[], filePath?: string | undefined) => Diagnostic[];
    readonly shouldReportDiagnostic: (filePath: string) => boolean;
    readonly shouldStringifyContent: (filePath: string) => boolean;
    readonly createTsError: (diagnostics: ReadonlyArray<Diagnostic>) => TSError;
    readonly tsCacheDir: string | undefined;
    readonly rootDir: string;
    readonly cwd: string;
    readonly isDoctoring: boolean;
    readTsConfig(compilerOptions?: object, project?: string | null, noProject?: boolean | null): ReadTsConfigResult;
    resolvePath(inputPath: string, noFailIfMissing?: boolean): string;
    readonly jsonValue: JsonableValue<{
        version: string;
        jest: {
            automock: boolean;
            browser: boolean;
            cache: boolean;
            cacheDirectory: string;
            clearMocks: boolean;
            coveragePathIgnorePatterns: string[];
            cwd: string;
            detectLeaks: boolean;
            displayName: jest.Maybe<string>;
            forceCoverageMatch: string[];
            globals: object;
            haste: jest.HasteConfig;
            moduleDirectories: string[];
            moduleFileExtensions: string[];
            moduleLoader: string;
            moduleNameMapper: [string, string][];
            modulePathIgnorePatterns: string[];
            modulePaths: string[];
            name: string;
            resetMocks: boolean;
            resetModules: boolean;
            resolver: jest.Maybe<string>;
            rootDir: string;
            roots: string[];
            runner: string;
            setupFiles: string[];
            setupTestFrameworkScriptFile: string;
            skipNodeResolution: boolean;
            snapshotSerializers: string[];
            testEnvironment: string;
            testEnvironmentOptions: object;
            testLocationInResults: boolean;
            testMatch: string[];
            testPathIgnorePatterns: string[];
            testRegex: string;
            testRunner: string;
            testURL: string;
            timers: "real" | "fake";
            transform: [string, string][];
            transformIgnorePatterns: string[];
            unmockedModulePathPatterns: jest.Maybe<string[]>;
            watchPathIgnorePatterns: string[];
        };
        tsJest: TsJestConfig;
        babel: import("babel__core").TransformOptions | undefined;
        tsconfig: any;
    }>;
    readonly cacheKey: string;
    toJSON(): {
        version: string;
        jest: {
            automock: boolean;
            browser: boolean;
            cache: boolean;
            cacheDirectory: string;
            clearMocks: boolean;
            coveragePathIgnorePatterns: string[];
            cwd: string;
            detectLeaks: boolean;
            displayName: jest.Maybe<string>;
            forceCoverageMatch: string[];
            globals: object;
            haste: jest.HasteConfig;
            moduleDirectories: string[];
            moduleFileExtensions: string[];
            moduleLoader: string;
            moduleNameMapper: [string, string][];
            modulePathIgnorePatterns: string[];
            modulePaths: string[];
            name: string;
            resetMocks: boolean;
            resetModules: boolean;
            resolver: jest.Maybe<string>;
            rootDir: string;
            roots: string[];
            runner: string;
            setupFiles: string[];
            setupTestFrameworkScriptFile: string;
            skipNodeResolution: boolean;
            snapshotSerializers: string[];
            testEnvironment: string;
            testEnvironmentOptions: object;
            testLocationInResults: boolean;
            testMatch: string[];
            testPathIgnorePatterns: string[];
            testRegex: string;
            testRunner: string;
            testURL: string;
            timers: "real" | "fake";
            transform: [string, string][];
            transformIgnorePatterns: string[];
            unmockedModulePathPatterns: jest.Maybe<string[]>;
            watchPathIgnorePatterns: string[];
        };
        tsJest: TsJestConfig;
        babel: import("babel__core").TransformOptions | undefined;
        tsconfig: any;
    };
}
export {};
