"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
/**
 * This is the core of settings and so ts-jest.
 * Since configuration are used to create a good cache key, everything
 * depending on it is here. Fast jest relies on correct cache keys
 * depending on all settings that could affect the generated output.
 *
 * The big issue is that jest calls first `getCacheKey()` with stringified
 * version of the `jest.ProjectConfig`, and then later it calls `process()`
 * with the complete, object version of it.
 */
var jsonable_value_1 = require("../util/jsonable-value");
var path_1 = require("path");
var memoize_1 = require("../util/memoize");
var backports_1 = require("../util/backports");
var messages_1 = require("../util/messages");
var json5_1 = tslib_1.__importDefault(require("json5"));
var fs_1 = require("fs");
var importer_1 = require("../util/importer");
var ts_error_1 = require("../util/ts-error");
var sha1_1 = require("../util/sha1");
var json_1 = require("../util/json");
var normalize_slashes_1 = require("../util/normalize-slashes");
var compiler_1 = require("../compiler");
var __1 = require("..");
var semver_1 = tslib_1.__importDefault(require("semver"));
var logger_1 = require("../util/logger");
var logger = logger_1.rootLogger.child({ namespace: 'config' });
// this regex MUST match nothing, it's the goal ;-)
exports.MATCH_NOTHING = /a^/;
exports.IGNORE_DIAGNOSTIC_CODES = [
    6059,
    18002,
    18003,
];
var DEFAULT_COMPILER_OPTIONS = {
    inlineSourceMap: true,
    inlineSources: true,
};
var FORCE_COMPILER_OPTIONS = {
    // we handle sourcemaps this way and not another
    sourceMap: true,
    inlineSourceMap: false,
    inlineSources: true,
    // we don't want to create declaration files
    declaration: false,
    noEmit: false,
    outDir: '$$ts-jest$$',
    // commonjs + module interop should be compatible with every other setup
    module: 'commonjs',
    esModuleInterop: true,
    // else istanbul related will be dropped
    removeComments: false,
    // to clear out else it's buggy
    out: undefined,
    outFile: undefined,
    composite: undefined,
    declarationDir: undefined,
    declarationMap: undefined,
    emitDeclarationOnly: undefined,
    sourceRoot: undefined,
};
var normalizeRegex = function (pattern) {
    return pattern
        ? typeof pattern === 'string'
            ? pattern
            : pattern.source
        : undefined;
};
var toDiagnosticCode = function (code) {
    return code
        ? parseInt(('' + code).trim().replace(/^TS/, ''), 10) || undefined
        : undefined;
};
var toDiagnosticCodeList = function (items, into) {
    if (into === void 0) { into = []; }
    var e_1, _a;
    if (!Array.isArray(items))
        items = [items];
    try {
        for (var items_1 = tslib_1.__values(items), items_1_1 = items_1.next(); !items_1_1.done; items_1_1 = items_1.next()) {
            var item = items_1_1.value;
            if (!item)
                continue;
            if (Array.isArray(item)) {
                toDiagnosticCodeList(item, into);
                continue;
            }
            else if (typeof item === 'string') {
                var children = item.trim().split(/\s*,\s*/g);
                if (children.length > 1) {
                    toDiagnosticCodeList(children, into);
                    continue;
                }
                item = children[0];
            }
            if (!item)
                continue;
            var code = toDiagnosticCode(item);
            if (code && !into.includes(code))
                into.push(code);
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (items_1_1 && !items_1_1.done && (_a = items_1.return)) _a.call(items_1);
        }
        finally { if (e_1) throw e_1.error; }
    }
    return into;
};
var ConfigSet = /** @class */ (function () {
    function ConfigSet(_jestConfig, parentOptions, parentLogger) {
        this._jestConfig = _jestConfig;
        this.parentOptions = parentOptions;
        this.logger = parentLogger
            ? parentLogger.child({ namespace: 'config' })
            : logger;
    }
    Object.defineProperty(ConfigSet.prototype, "jest", {
        get: function () {
            var config = backports_1.backportJestConfig(this.logger, this._jestConfig);
            if (this.parentOptions) {
                var globals = config.globals || (config.globals = {});
                // TODO: implement correct deep merging instead
                globals['ts-jest'] = tslib_1.__assign({}, this.parentOptions, globals['ts-jest']);
            }
            this.logger.debug({ jestConfig: config }, 'normalized jest config');
            return config;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ConfigSet.prototype, "tsJest", {
        get: function () {
            var parsedConfig = this.jest;
            var _a = parsedConfig.globals, globals = _a === void 0 ? {} : _a;
            var options = tslib_1.__assign({}, globals['ts-jest']);
            // tsconfig
            var tsConfigOpt = options.tsConfig;
            var tsConfig;
            if (typeof tsConfigOpt === 'string' ||
                tsConfigOpt == null ||
                tsConfigOpt === true) {
                tsConfig = {
                    kind: 'file',
                    value: typeof tsConfigOpt === 'string'
                        ? this.resolvePath(tsConfigOpt)
                        : undefined,
                };
            }
            else if (typeof tsConfigOpt === 'object') {
                tsConfig = {
                    kind: 'inline',
                    value: tsConfigOpt,
                };
            }
            // babel jest
            var babelConfigOpt = options.babelConfig;
            var babelConfig;
            if (typeof babelConfigOpt === 'string' || babelConfigOpt === true) {
                babelConfig = {
                    kind: 'file',
                    value: babelConfigOpt === true
                        ? undefined
                        : this.resolvePath(babelConfigOpt),
                };
            }
            else if (babelConfigOpt) {
                babelConfig = {
                    kind: 'inline',
                    value: babelConfigOpt,
                };
            }
            // diagnostics
            var diagnostics;
            var _b = options.diagnostics, diagnosticsOpt = _b === void 0 ? true : _b;
            // messy list of stuff to ignore (will be casted later)
            var ignoreList = [
                exports.IGNORE_DIAGNOSTIC_CODES,
                process.env.TS_JEST_IGNORE_DIAGNOSTICS,
            ];
            if (diagnosticsOpt === true || diagnosticsOpt == null) {
                diagnostics = { ignoreCodes: [], pretty: true };
            }
            else if (diagnosticsOpt === false) {
                diagnostics = {
                    pretty: true,
                    ignoreCodes: [],
                    pathRegex: exports.MATCH_NOTHING.source,
                };
            }
            else {
                ignoreList.push(diagnosticsOpt.ignoreCodes);
                diagnostics = {
                    pretty: diagnosticsOpt.pretty == null ? true : !!diagnosticsOpt.pretty,
                    ignoreCodes: [],
                    pathRegex: normalizeRegex(diagnosticsOpt.pathRegex),
                };
            }
            // now we clean and flaten the list
            diagnostics.ignoreCodes = toDiagnosticCodeList(ignoreList);
            // stringifyContentPathRegex option
            var stringifyContentPathRegex = normalizeRegex(options.stringifyContentPathRegex);
            // parsed options
            var res = {
                tsConfig: tsConfig,
                babelConfig: babelConfig,
                diagnostics: diagnostics,
                typeCheck: !!options.typeCheck,
                compiler: options.compiler || 'typescript',
                stringifyContentPathRegex: stringifyContentPathRegex,
            };
            this.logger.debug({ tsJestConfig: res }, 'normalized ts-jest config');
            return res;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ConfigSet.prototype, "typescript", {
        get: function () {
            return this._typescript.resolved;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ConfigSet.prototype, "tsconfig", {
        get: function () {
            return this._typescript.input;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ConfigSet.prototype, "_typescript", {
        get: function () {
            var tsConfig = this.tsJest.tsConfig;
            var result = this.readTsConfig(tsConfig && tsConfig.kind === 'inline' ? tsConfig.value : undefined, tsConfig && tsConfig.kind === 'file' ? tsConfig.value : undefined, tsConfig == null);
            // throw errors if any matching wanted diagnostics
            var configDiagnosticList = this.filterDiagnostics(result.resolved.errors);
            if (configDiagnosticList.length) {
                throw this.createTsError(configDiagnosticList);
            }
            this.logger.debug({ tsconfig: result }, 'normalized typescript config');
            return result;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ConfigSet.prototype, "babel", {
        get: function () {
            var babelConfig = this.tsJest.babelConfig;
            if (babelConfig == null) {
                this.logger.debug('babel is disabled');
                return;
            }
            var base = { cwd: this.cwd };
            if (babelConfig.kind === 'file') {
                if (babelConfig.value) {
                    base = tslib_1.__assign({}, base, json5_1.default.parse(fs_1.readFileSync(babelConfig.value, 'utf8')));
                }
            }
            else if (babelConfig.kind === 'inline') {
                base = tslib_1.__assign({}, base, babelConfig.value);
            }
            // loadOptions is from babel 7+, and OptionManager is backward compatible but deprecated 6 API
            var _a = importer_1.importer.babelCore(messages_1.ImportReasons.BabelJest), OptionManager = _a.OptionManager, loadOptions = _a.loadOptions, version = _a.version;
            // cwd is only supported from babel >= 7
            if (version && semver_1.default.satisfies(version, '>=6 <7')) {
                delete base.cwd;
            }
            // call babel to load options
            var config;
            if (typeof loadOptions === 'function') {
                config = loadOptions(base);
            }
            else {
                config = new OptionManager().init(base);
            }
            this.logger.debug({ babelConfig: config }, 'normalized babel config');
            return config;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ConfigSet.prototype, "compilerModule", {
        get: function () {
            return importer_1.importer.typescript(messages_1.ImportReasons.TsJest, this.tsJest.compiler);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ConfigSet.prototype, "babelJestTransformer", {
        get: function () {
            var babel = this.babel;
            if (!babel)
                return;
            this.logger.debug('creating babel-jest transformer');
            return importer_1.importer
                .babelJest(messages_1.ImportReasons.BabelJest)
                .createTransformer(babel);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ConfigSet.prototype, "tsCompiler", {
        get: function () {
            return compiler_1.createCompiler(this);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ConfigSet.prototype, "hooks", {
        get: function () {
            var hooksFile = process.env.TS_JEST_HOOKS;
            if (hooksFile) {
                hooksFile = path_1.resolve(this.cwd, hooksFile);
                return importer_1.importer.tryThese(hooksFile) || {};
            }
            return {};
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ConfigSet.prototype, "filterDiagnostics", {
        get: function () {
            var _a = this, ignoreCodes = _a.tsJest.diagnostics.ignoreCodes, shouldReportDiagnostic = _a.shouldReportDiagnostic;
            return function (diagnostics, filePath) {
                if (filePath && !shouldReportDiagnostic(filePath))
                    return [];
                return diagnostics.filter(function (diagnostic) {
                    if (diagnostic.file &&
                        diagnostic.file.fileName &&
                        !shouldReportDiagnostic(diagnostic.file.fileName)) {
                        return false;
                    }
                    return ignoreCodes.indexOf(diagnostic.code) === -1;
                });
            };
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ConfigSet.prototype, "shouldReportDiagnostic", {
        get: function () {
            var pathRegex = this.tsJest.diagnostics.pathRegex;
            if (pathRegex) {
                var regex_1 = new RegExp(pathRegex);
                return function (file) { return regex_1.test(file); };
            }
            else {
                return function () { return true; };
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ConfigSet.prototype, "shouldStringifyContent", {
        get: function () {
            var stringifyContentPathRegex = this.tsJest.stringifyContentPathRegex;
            if (stringifyContentPathRegex) {
                var regex_2 = new RegExp(stringifyContentPathRegex);
                return function (file) { return regex_2.test(file); };
            }
            else {
                return function () { return false; };
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ConfigSet.prototype, "createTsError", {
        get: function () {
            var _this = this;
            var pretty = this.tsJest.diagnostics.pretty;
            var formatDiagnostics = pretty
                ? this.compilerModule.formatDiagnosticsWithColorAndContext
                : this.compilerModule.formatDiagnostics;
            var diagnosticHost = {
                getNewLine: function () { return '\n'; },
                getCurrentDirectory: function () { return _this.cwd; },
                getCanonicalFileName: function (path) { return path; },
            };
            return function (diagnostics) {
                var diagnosticText = formatDiagnostics(diagnostics, diagnosticHost);
                var diagnosticCodes = diagnostics.map(function (x) { return x.code; });
                return new ts_error_1.TSError(diagnosticText, diagnosticCodes);
            };
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ConfigSet.prototype, "tsCacheDir", {
        get: function () {
            if (!this.jest.cache) {
                logger.debug('file caching disabled');
                return;
            }
            var cacheSufix = sha1_1.sha1(json_1.stringify({
                version: this.compilerModule.version,
                compiler: this.tsJest.compiler,
                compilerOptions: this.typescript.options,
                typeCheck: this.tsJest.typeCheck,
                ignoreDiagnostics: this.tsJest.diagnostics.ignoreCodes,
            }));
            var res = path_1.join(this.jest.cacheDirectory, "ts-jest-" + cacheSufix);
            logger.debug({ cacheDirectory: res }, "will use file caching");
            return res;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ConfigSet.prototype, "rootDir", {
        get: function () {
            return this.jest.rootDir || this.cwd;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ConfigSet.prototype, "cwd", {
        get: function () {
            return this.jest.cwd || process.cwd();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ConfigSet.prototype, "isDoctoring", {
        get: function () {
            return !!process.env.TS_JEST_DOCTOR;
        },
        enumerable: true,
        configurable: true
    });
    ConfigSet.prototype.readTsConfig = function (compilerOptions, project, noProject) {
        var config = { compilerOptions: {} };
        var basePath = normalize_slashes_1.normalizeSlashes(this.cwd);
        var configFileName;
        var ts = this.compilerModule;
        var input;
        if (noProject) {
            input = { compilerOptions: tslib_1.__assign({}, compilerOptions) };
        }
        else {
            // Read project configuration when available.
            configFileName = project
                ? normalize_slashes_1.normalizeSlashes(path_1.resolve(this.cwd, project))
                : ts.findConfigFile(normalize_slashes_1.normalizeSlashes(this.cwd), ts.sys.fileExists);
            if (configFileName) {
                this.logger.debug({ tsConfigFileName: configFileName }, 'readTsConfig(): reading', configFileName);
                var result_1 = ts.readConfigFile(configFileName, ts.sys.readFile);
                // Return diagnostics.
                if (result_1.error) {
                    return {
                        resolved: { errors: [result_1.error], fileNames: [], options: {} },
                    };
                }
                config = result_1.config;
                input = tslib_1.__assign({}, result_1.config, { compilerOptions: tslib_1.__assign({}, (result_1.config && result_1.config.compilerOptions), compilerOptions) });
                basePath = normalize_slashes_1.normalizeSlashes(path_1.dirname(configFileName));
            }
        }
        // Override default configuration options `ts-jest` requires.
        config.compilerOptions = tslib_1.__assign({}, DEFAULT_COMPILER_OPTIONS, config.compilerOptions, compilerOptions, FORCE_COMPILER_OPTIONS);
        var result = ts.parseJsonConfigFileContent(config, ts.sys, basePath, undefined, configFileName);
        // Target ES5 output by default (instead of ES3).
        if (result.options.target === undefined) {
            result.options.target = ts.ScriptTarget.ES5;
        }
        // ensure undefined in FORCE_COMPILER_OPTIONS are removed
        for (var key in FORCE_COMPILER_OPTIONS) {
            if (FORCE_COMPILER_OPTIONS[key] === undefined) {
                delete result.options[key];
            }
        }
        return { input: input, resolved: result };
    };
    ConfigSet.prototype.resolvePath = function (inputPath, noFailIfMissing) {
        if (noFailIfMissing === void 0) { noFailIfMissing = false; }
        var path = inputPath;
        if (path.startsWith('<rootDir>')) {
            path = path_1.resolve(this.rootDir, path.substr(9));
        }
        else if (!path_1.isAbsolute(path)) {
            path = path_1.resolve(this.cwd, path);
        }
        if (!noFailIfMissing && !fs_1.existsSync(path)) {
            throw new Error(messages_1.interpolate(messages_1.Errors.FileNotFound, { inputPath: inputPath, resolvedPath: path }));
        }
        this.logger.debug({ fromPath: inputPath, toPath: path }, 'resolved path from', inputPath, 'to', path);
        return path;
    };
    Object.defineProperty(ConfigSet.prototype, "jsonValue", {
        get: function () {
            var jest = tslib_1.__assign({}, this.jest);
            var globals = (jest.globals = tslib_1.__assign({}, jest.globals));
            // we need to remove some stuff from jest config
            // this which does not depend on config
            delete jest.name;
            delete jest.cacheDirectory;
            // we do not need this since its normalized version is in tsJest
            delete globals['ts-jest'];
            return new jsonable_value_1.JsonableValue({
                version: __1.version,
                jest: jest,
                tsJest: this.tsJest,
                babel: this.babel,
                tsconfig: this.tsconfig,
            });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ConfigSet.prototype, "cacheKey", {
        get: function () {
            return this.jsonValue.serialized;
        },
        enumerable: true,
        configurable: true
    });
    ConfigSet.prototype.toJSON = function () {
        return this.jsonValue.value;
    };
    tslib_1.__decorate([
        memoize_1.Memoize()
    ], ConfigSet.prototype, "jest", null);
    tslib_1.__decorate([
        memoize_1.Memoize()
    ], ConfigSet.prototype, "tsJest", null);
    tslib_1.__decorate([
        memoize_1.Memoize()
    ], ConfigSet.prototype, "_typescript", null);
    tslib_1.__decorate([
        memoize_1.Memoize()
    ], ConfigSet.prototype, "babel", null);
    tslib_1.__decorate([
        memoize_1.Memoize()
    ], ConfigSet.prototype, "compilerModule", null);
    tslib_1.__decorate([
        memoize_1.Memoize()
    ], ConfigSet.prototype, "babelJestTransformer", null);
    tslib_1.__decorate([
        memoize_1.Memoize()
    ], ConfigSet.prototype, "tsCompiler", null);
    tslib_1.__decorate([
        memoize_1.Memoize()
    ], ConfigSet.prototype, "hooks", null);
    tslib_1.__decorate([
        memoize_1.Memoize()
    ], ConfigSet.prototype, "filterDiagnostics", null);
    tslib_1.__decorate([
        memoize_1.Memoize()
    ], ConfigSet.prototype, "shouldReportDiagnostic", null);
    tslib_1.__decorate([
        memoize_1.Memoize()
    ], ConfigSet.prototype, "shouldStringifyContent", null);
    tslib_1.__decorate([
        memoize_1.Memoize()
    ], ConfigSet.prototype, "createTsError", null);
    tslib_1.__decorate([
        memoize_1.Memoize()
    ], ConfigSet.prototype, "tsCacheDir", null);
    tslib_1.__decorate([
        memoize_1.Memoize()
    ], ConfigSet.prototype, "jsonValue", null);
    return ConfigSet;
}());
exports.ConfigSet = ConfigSet;
//# sourceMappingURL=config-set.js.map