"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var bs_logger_1 = require("bs-logger");
/**
 * What methods of `jest` should we hoist
 */
var HOIST_METHODS = ['mock', 'unmock'];
/**
 * The factory of hoisting transformer factory
 * @param cs Current jest configuration-set
 */
function factory(cs) {
    var logger = cs.logger.child({ namespace: 'ts-hoisting' });
    /**
     * Our compiler (typescript, or a module with typescript-like interface)
     */
    var ts = cs.compilerModule;
    /**
     * Checks whether given node is a statement that we need to hoist
     * @param node The node to test
     */
    function shouldHoistNode(node) {
        return (ts.isExpressionStatement(node) &&
            ts.isCallExpression(node.expression) &&
            ts.isPropertyAccessExpression(node.expression.expression) &&
            ts.isIdentifier(node.expression.expression.expression) &&
            node.expression.expression.expression.text === 'jest' &&
            ts.isIdentifier(node.expression.expression.name) &&
            HOIST_METHODS.includes(node.expression.expression.name.text));
    }
    /**
     * Create a source file visitor which will visit all nodes in a source file
     * @param ctx The typescript transformation context
     * @param sf The owning source file
     */
    function createVisitor(ctx, sf) {
        /**
         * Current block level
         */
        var level = 0;
        /**
         * List of nodes which needs to be hoisted, indexed by their owning level
         */
        var hoisted = [];
        /**
         * Called when we enter a block to increase the level
         */
        var enter = function () {
            level++;
            // reuse arrays
            if (hoisted[level]) {
                hoisted[level].splice(0, hoisted[level].length);
            }
        };
        /**
         * Called when we leave a block to devrease the level
         */
        var exit = function () { return level--; };
        /**
         * Adds a node to the list of nodes to be hoisted in the current level
         * @param node The node to hoist
         */
        var hoist = function (node) {
            if (hoisted[level]) {
                hoisted[level].push(node);
            }
            else {
                hoisted[level] = [node];
            }
        };
        /**
         * Our main visitor, which will be called recursively for each node in the source file's AST
         * @param node The node to be visited
         */
        var visitor = function (node) {
            // enter this level
            enter();
            // visit each child
            var resultNode = ts.visitEachChild(node, visitor, ctx);
            // check if we have something to hoist in this level
            if (hoisted[level] && hoisted[level].length) {
                // re-order children so that hoisted ones appear first
                // this is actually the main work of this transformer
                var block = resultNode;
                block.statements = ts.createNodeArray(tslib_1.__spread(hoisted[level], block.statements));
            }
            // exit the level
            exit();
            if (shouldHoistNode(resultNode)) {
                // hoist into current level
                hoist(resultNode);
                return;
            }
            // finsally returns the currently visited node
            return resultNode;
        };
        return visitor;
    }
    // returns the transformer factory
    return function (ctx) {
        var _a;
        return logger.wrap((_a = {}, _a[bs_logger_1.LogContexts.logLevel] = bs_logger_1.LogLevels.debug, _a.call = null, _a), 'visitSourceFileNode(): hoisting', function (sf) { return ts.visitNode(sf, createVisitor(ctx, sf)); });
    };
}
exports.factory = factory;
//# sourceMappingURL=hoisting.js.map