"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var memoize_1 = require("./memoize");
var hacks = tslib_1.__importStar(require("./hacks"));
var messages_1 = require("./messages");
var version_checkers_1 = require("./version-checkers");
var logger_1 = require("./logger");
var logger = logger_1.rootLogger.child({ namespace: 'Importer' });
var passThru = function (action) { return function (input) {
    action();
    return input;
}; };
var Importer = /** @class */ (function () {
    function Importer(_patches) {
        if (_patches === void 0) { _patches = {}; }
        this._patches = _patches;
    }
    Object.defineProperty(Importer, "instance", {
        get: function () {
            logger.debug('creating Importer singleton');
            // here we can define patches to apply to modules.
            // it could be fixes that are not deployed, or
            // abstractions so that multiple versions work the same
            return new Importer({
                'babel-core': [
                    passThru(version_checkers_1.VersionCheckers.babelCoreLegacy.warn),
                    hacks.patchBabelCore_githubIssue6577,
                ],
                '@babel/core': [passThru(version_checkers_1.VersionCheckers.babelCore.warn)],
                'babel-jest': [passThru(version_checkers_1.VersionCheckers.babelJest.warn)],
                typescript: [passThru(version_checkers_1.VersionCheckers.typescript.warn)],
                jest: [passThru(version_checkers_1.VersionCheckers.jest.warn)],
            });
        },
        enumerable: true,
        configurable: true
    });
    Importer.prototype.babelJest = function (why) {
        // this is to ensure babel-core is patched
        this.tryThese('babel-core');
        return this._import(why, 'babel-jest');
    };
    Importer.prototype.babelCore = function (why) {
        // the bridge will choose correct babel version
        return this._import(why, 'babel-core', {
            alternatives: ['@babel/core'],
            installTip: [
                // as in https://github.com/facebook/jest/tree/master/packages/babel-jest
                {
                    label: 'for Babel 7',
                    module: "babel-jest 'babel-core@^7.0.0-0' @babel/core",
                },
                { label: 'for Babel 6', module: 'babel-jest babel-core' },
            ],
        });
    };
    Importer.prototype.typescript = function (why, which) {
        return this._import(why, which);
    };
    Importer.prototype.tryThese = function (moduleName) {
        var fallbacks = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            fallbacks[_i - 1] = arguments[_i];
        }
        var name;
        var loaded;
        var tries = tslib_1.__spread([moduleName], fallbacks);
        // tslint:disable-next-line:no-conditional-assignment
        while ((name = tries.shift()) !== undefined) {
            try {
                loaded = requireModule(name);
                logger.debug('loaded module', name);
                break;
            }
            catch (err) {
                logger.debug('fail loading module', name);
            }
        }
        return loaded && this._patch(name, loaded);
    };
    Importer.prototype._patch = function (name, unpatched) {
        if (name in this._patches) {
            logger.debug('patching', name);
            return this._patches[name].reduce(function (mod, patcher) { return patcher(mod); }, unpatched);
        }
        return unpatched;
    };
    Importer.prototype._import = function (why, moduleName, _a) {
        var _b = _a === void 0 ? {} : _a, _c = _b.alternatives, alternatives = _c === void 0 ? [] : _c, _d = _b.installTip, installTip = _d === void 0 ? moduleName : _d;
        // try to load any of the alternative after trying main one
        var res = this.tryThese.apply(this, tslib_1.__spread([moduleName], alternatives));
        // if we could load one, return it
        if (res) {
            return res;
        }
        // if it couldn't load, build a nice error message so the user can fix it by himself
        var msg = alternatives.length
            ? messages_1.Errors.UnableToLoadAnyModule
            : messages_1.Errors.UnableToLoadOneModule;
        var loadModule = tslib_1.__spread([moduleName], alternatives).map(function (m) { return "\"" + m + "\""; })
            .join(', ');
        if (typeof installTip === 'string') {
            installTip = [{ module: installTip, label: "install \"" + installTip + "\"" }];
        }
        var fix = installTip
            .map(function (tip) {
            return "    " + (installTip.length === 1 ? '↳' : '•') + " " + messages_1.interpolate(messages_1.Helps.FixMissingModule, tip);
        })
            .join('\n');
        throw new Error(messages_1.interpolate(msg, {
            module: loadModule,
            reason: why,
            fix: fix,
        }));
    };
    tslib_1.__decorate([
        memoize_1.Memoize(function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            return args.join(':');
        })
    ], Importer.prototype, "tryThese", null);
    tslib_1.__decorate([
        memoize_1.Memoize(function (name) { return name; })
    ], Importer.prototype, "_patch", null);
    tslib_1.__decorate([
        memoize_1.Memoize()
    ], Importer, "instance", null);
    return Importer;
}());
exports.Importer = Importer;
exports.importer = Importer.instance;
var requireModule = function (mod) { return require(mod); };
// so that we can test easier
function __requireModule(localRequire) {
    requireModule = localRequire;
}
exports.__requireModule = __requireModule;
//# sourceMappingURL=importer.js.map